using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using Android.App;
using Android.Content;
using Android.OS;
using Android.Runtime;
using Android.Views;
using Android.Widget;
using Com.Syncfusion.Charts;
using Android.Graphics;
using Android.Views.Animations;
using System.Text.RegularExpressions;

namespace SimpleSample.Droid
{
    public class CustomTrackball : ChartTrackballBehavior
    {
        Rect textRect;
        RectAlignment alignment;
        
        public CustomTrackball()
        {
           textRect = new Rect();
           alignment = RectAlignment.Top;
        }

        protected override void DrawLabel(Android.Graphics.Canvas canvas, ChartPointInfo pointInfo, Android.Graphics.RectF rect)
        {
            PointF intersectPoint1;
            PointF intersectPoint2;
            float xPosition, yPosition;
            xPosition = pointInfo.XPosition;
            yPosition = pointInfo.YPosition;

            bool isDotSeries = (pointInfo.Series is BubbleSeries) || (pointInfo.Series is ScatterSeries);
            Size size = MeasureMultiLineLabel(pointInfo.Label, pointInfo.LabelStyle);
            GetTextPaint(pointInfo.LabelStyle).GetTextBounds(pointInfo.Label, 0, pointInfo.Label.Length, textRect);

            if (MarkerStyle.ShowMarker && !isDotSeries) {
              rect.Set(pointInfo.XPosition - size.Width / 2 - pointInfo.LabelStyle.MarginLeft, 
                    yPosition - 10 - size.Height - pointInfo.LabelStyle.MarginTop,
                    pointInfo.XPosition + size.Width / 2 + pointInfo.LabelStyle.MarginRight,
                    yPosition - 10 + pointInfo.LabelStyle.MarginBottom);
            }

            //Checking the coll
            rect = CalculateBounds(rect, xPosition, yPosition);
            if (alignment== RectAlignment.Left)
            {
                if (MarkerStyle.ShowMarker && !isDotSeries)
                {
                    xPosition -= MarkerStyle.Width / 2;
                    rect.Set(rect.Left - MarkerStyle.Width / 2, rect.Top, rect.Right - MarkerStyle.Width / 2, rect.Bottom);
                }
                intersectPoint1 = new PointF(rect.Right, rect.CenterY() - 5);
                intersectPoint2 = new PointF(rect.Right, rect.CenterY() + 5);
                xPosition -= 2;
            }
            else if (alignment== RectAlignment.Right)
            {
                if (MarkerStyle.ShowMarker && !isDotSeries)
                {
                    xPosition += MarkerStyle.Width / 2;
                    rect.Set(rect.Left + MarkerStyle.Width / 2, rect.Top, rect.Right + MarkerStyle.Width / 2, rect.Bottom);
                }
                intersectPoint1 = new PointF(rect.Left, rect.CenterY() - 5);
                intersectPoint2 = new PointF(rect.Left, rect.CenterY() + 5);
                xPosition += 2;
            }
            else if (alignment== RectAlignment.Top)
            {
                if (MarkerStyle.ShowMarker && !isDotSeries)
                {
                    yPosition -= MarkerStyle.Height / 2;
                    rect.Set(rect.Left, rect.Top - MarkerStyle.Height / 2, rect.Right, rect.Bottom - MarkerStyle.Height / 2);
                }
                intersectPoint1 = new PointF(rect.CenterX() - 5, rect.Bottom);
                intersectPoint2 = new PointF(rect.CenterX() + 5, rect.Bottom);
                yPosition -= 2;
            }
            else
            {
                if (MarkerStyle.ShowMarker && !isDotSeries)
                {
                    yPosition += MarkerStyle.Height / 2;
                    rect.Set(rect.Left, rect.Top + MarkerStyle.Height / 2, rect.Right, rect.Bottom + MarkerStyle.Height / 2);
                }
                intersectPoint1 = new PointF(rect.CenterX() - 5, rect.Top);
                intersectPoint2 = new PointF(rect.CenterX() + 5, rect.Top);
                yPosition += 2;
            }
            //    pointInfo.YPosition -= 2;
     
            //To draw the strokePath and Background
            Path backgroundPath = new Path();
          
            backgroundPath.Reset();
            backgroundPath.MoveTo(intersectPoint1.X, intersectPoint1.Y);
            backgroundPath.LineTo(  pointInfo.XPosition, pointInfo.YPosition);
            backgroundPath.LineTo(intersectPoint2.X, intersectPoint2.Y);
            backgroundPath.LineTo(intersectPoint1.X, intersectPoint1.Y);

            backgroundPath.AddRoundRect(rect, 3, 3, Android.Graphics.Path.Direction.Ccw);
            backgroundPath.Close();
               
            canvas.DrawPath(backgroundPath, GetStrokePaint(pointInfo.LabelStyle));
            canvas.DrawPath(backgroundPath, GetBackgroundPaint(pointInfo.LabelStyle));

             //To draw the label text
             DrawLabel(canvas, pointInfo.Label, rect, pointInfo.LabelStyle);
            
        }

        private void DrawLabel(Canvas canvas, string label, RectF rect, ChartLabelStyle labelStyle)
        {
            float x, y;
            x = rect.CenterX();
            y = rect.CenterY();
            
            if (label.Contains("\n"))
            {
                Size size = MeasureMultiLineLabel(label, labelStyle);
                DrawMultiLineLabel(canvas, label, labelStyle, x - size.Width / 2, y - size.Height / 2);
            }
        }

        Paint GetStrokePaint(ChartLabelStyle labelStyle)
        {
            Paint strokePaint = new Paint();
            strokePaint.AntiAlias = true;
            strokePaint.SetStyle(Android.Graphics.Paint.Style.Stroke);
            strokePaint.StrokeWidth = labelStyle.StrokeWidth;
            strokePaint.Color = new Color(labelStyle.StrokeColor);
            return strokePaint;
        }

        Paint GetBackgroundPaint(ChartLabelStyle labelStyle)
        {
       
            Paint backgroundPaint = new Paint();
            backgroundPaint.SetStyle(Android.Graphics.Paint.Style.Fill);
            backgroundPaint.AntiAlias = true;
            backgroundPaint.Color = new Color(labelStyle.BackgroundColor);
            return backgroundPaint;
        }

        Paint GetTextPaint(ChartLabelStyle labelStyle)
        {
            Paint textPaint = new Paint();
            textPaint.SetStyle(Android.Graphics.Paint.Style.Fill);
            textPaint.AntiAlias = true;
            textPaint.Color = new Color(labelStyle.TextColor);
            textPaint.TextSize = labelStyle.TextSize * Chart.Context.Resources.DisplayMetrics.Density;
            return textPaint;
        }

        void DrawMultiLineLabel(Canvas canvas, String label, ChartLabelStyle labelStyle, float x, float y)
        {
            foreach (string line in Regex.Split(label, "\n"))
            {
                GetTextPaint(labelStyle).GetTextBounds(line, 0, line.Length, textRect);
                canvas.DrawText(line, x, y + textRect.Height() + labelStyle.MarginTop, GetTextPaint(labelStyle));
                y += GetTextPaint(labelStyle).Descent() - GetTextPaint(labelStyle).Ascent() + labelStyle.MarginBottom;
            }
        }
            
         Size MeasureMultiLineLabel(String label, ChartLabelStyle labelStyle) 
         {
            float height = 0, Width = 0;

            foreach (string line in Regex.Split(label, "\n"))
            {
                GetTextPaint(labelStyle).GetTextBounds(line, 0, line.Length, textRect);
                if (Width < textRect.Width()) Width = textRect.Width() + labelStyle.MarginLeft + labelStyle.MarginRight;
                height += textRect.Height() + labelStyle.MarginTop + labelStyle.MarginBottom;
            }
            return new Size(Width, height);
         }

         private RectF CalculateBounds(RectF rect, float xPosition, float yPosition)
         {
             RectF seriesBounds = Chart.SeriesBounds;
            
            if (rect.Top < Chart.SeriesBounds.Top)
            {
                alignment = (alignment == RectAlignment.Top) ? RectAlignment.Bottom : alignment;
                rect.Set(rect.Left, yPosition + 10, rect.Right, yPosition + 10 + rect.Height());
            }
            if (rect.Bottom > Chart.SeriesBounds.Bottom)
            {
                alignment = (alignment == RectAlignment.Bottom) ? RectAlignment.Top : alignment;
                rect.Set(rect.Left, yPosition - 10 - rect.Height(), rect.Right, yPosition - 10);
            }
            if (rect.Right > Chart.SeriesBounds.Right)
            {
                alignment = (alignment == RectAlignment.Right) ? RectAlignment.Left : alignment;
                rect.Set(xPosition - 10 - rect.Width(), rect.Top, xPosition - 10, rect.Bottom);
            }
            if (rect.Left < Chart.SeriesBounds.Left)
            {
                alignment = (alignment == RectAlignment.Left) ? RectAlignment.Right : alignment;
                rect.Set(xPosition + 10, rect.Top, xPosition + 10 + rect.Width(), rect.Bottom);
            }
            return rect;
         }
    }

    enum RectAlignment
    {
        Left,
        Top,
        Right,
        Bottom,
    }
}